@@ -1524,11 +1524,14 @@ const char *default_enhancement_xpm[] =
 const char *pipette_white_xpm[] =
 {
 /* width height num_colors chars_per_pixel */
-"    20    20        3            1",
+"    20    20        6            1",
 /* colors */
 ". c #000000",
 "  none",
 "a c #ffffff",
+"r c #af8f8f",
+"g c #8fff8f",
+"b c #8f8fff",
 /* pixels */
 "                    ",
 "                    ",
@@ -1537,18 +1540,18 @@ const char *pipette_white_xpm[] =
 "             ...... ",
 "          ......... ",
 "            ......  ",
-"           .aaa..   ",
-"          .aaaa..   ",
-"         .aaaa. .   ",
-"        .aaaa.  .   ",
-"       .aaaa.       ",
-"      .aaaa.        ",
-"     .aaaa.         ",
-"    .aaaa.          ",
-"   .aaaa.           ",
-"  .aaaa.            ",
-" .aaaa.             ",
-" .aaa.              ",
+"           .abg..   ",
+"          .abgr..   ",
+"         .abgr. .   ",
+"        .abgr.  .   ",
+"       .abgr.       ",
+"      .abgr.        ",
+"     .abgr.         ",
+"    .abgr.          ",
+"   .abgr.           ",
+"  .abgr.            ",
+" .abgr.             ",
+" .bgr.              ",
 "  ...               ",
 };
 
@@ -1557,12 +1560,15 @@ const char *pipette_white_xpm[] =
 const char *pipette_gray_xpm[] =
 {
 /* width height num_colors chars_per_pixel */
-"    20    20        4            1",
+"    20    20        7            1",
 /* colors */
 ". c #000000",
 "  none",
 "a c #ffffff",
 "X c #808080",
+"r c #af8f8f",
+"g c #8fff8f",
+"b c #8f8fff",
 /* pixels */
 "                    ",
 "                    ",
@@ -1571,9 +1577,9 @@ const char *pipette_gray_xpm[] =
 "             ...... ",
 "          ......... ",
 "            ......  ",
-"           .aaa..   ",
-"          .aaaa..   ",
-"         .aaaa. .   ",
+"           .abg..   ",
+"          .abgr..   ",
+"         .abgr. .   ",
 "        .XXXX.  .   ",
 "       .XXXX.       ",
 "      .XXXX.        ",
@@ -1590,11 +1596,14 @@ const char *pipette_gray_xpm[] =
 const char *pipette_black_xpm[] =
 {
 /* width height num_colors chars_per_pixel */
-"    20    20        3            1",
+"    20    20        6            1",
 /* colors */
 ". c #000000",
 "  none",
 "a c #ffffff",
+"r c #af8f8f",
+"g c #8fff8f",
+"b c #8f8fff",
 /* pixels */
 "                    ",
 "                    ",
@@ -1603,9 +1612,9 @@ const char *pipette_black_xpm[] =
 "             ...... ",
 "          ......... ",
 "            ......  ",
-"           .aaa..   ",
-"          .aaaa..   ",
-"         .aaaa. .   ",
+"           .abg..   ",
+"          .abgr..   ",
+"         .abgr. .   ",
 "        ......  .   ",
 "       ......       ",
 "      ......        ",
@@ -1620,27 +1629,67 @@ const char *pipette_black_xpm[] =
 
 /* --------------------------------------------------- */
 
-const char *zoom_in_xpm[] =
+const char *zoom_area_xpm[] =
 {
 /* width height num_colors chars_per_pixel */
-"    20    20        4            1",
+"    20    20        7            1",
 /* colors */
 "  none",
 ". c #000000",
 "+ c #c0c0c0",
 "a c #ffffff",
+"r c #af8f8f",
+"g c #8fff8f",
+"b c #8f8fff",
 /* pixels */
 "                    ",
 "                    ",
 "     .....          ",
 "    .+aaaa.         ",
-"   .++aaaaa.        ",
-"  .+++aaaaaa.       ",
-"  .aaaaaaaaa.       ",
-"  .aaaaaaaaa.       ",
-"  .aaaaaaaaa.       ",
-"   .aaaaaaa.        ",
-"    .aaaaa.         ",
+"   .++aaaag.        ",
+"  .+++aabagr.       ",
+"  .aaaaabbgr.       ",
+"  .aabbbbbgr.       ",
+"  .aaaabbgrr.       ",
+"   .gggggrr.        ",
+"    .rrrrr.         ",
+"     .......        ",
+"          ...       ",
+"           ...      ",
+" . . . .    ...     ",
+" .     .     ...    ",
+" .     .      ...   ",
+" .     .       ..   ",
+" . . . .            ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
+const char *zoom_in_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        7            1",
+/* colors */
+"  none",
+". c #000000",
+"+ c #c0c0c0",
+"a c #ffffff",
+"r c #af8f8f",
+"g c #8fff8f",
+"b c #8f8fff",
+/* pixels */
+"                    ",
+"                    ",
+"     .....          ",
+"    .+aaaa.         ",
+"   .++aaaag.        ",
+"  .+++aabagr.       ",
+"  .aaaaabbgr.       ",
+"  .aabbbbbgr.       ",
+"  .aaaabbgrr.       ",
+"   .gggggrr.        ",
+"    .rrrrr.         ",
 "     .......        ",
 "          ...       ",
 "    .      ...      ",
@@ -1657,24 +1706,27 @@ const char *zoom_in_xpm[] =
 const char *zoom_not_xpm[] =
 {
 /* width height num_colors chars_per_pixel */
-"    20    20        4            1",
+"    20    20        7            1",
 /* colors */
 "  none",
 ". c #000000",
 "+ c #c0c0c0",
 "a c #ffffff",
+"r c #af8f8f",
+"g c #8fff8f",
+"b c #8f8fff",
 /* pixels */
 "                    ",
 "                    ",
 " ..  .....       .. ",
 "  ...+aaaa.     ..  ",
-"   ..+aaaaa.   ..   ",
-"  .+..aaaaaa. ..    ",
-"  .aa..aaaaa...     ",
-"  .aaa..aaaa..      ",
-"  .aaaa..aa..       ",
-"   .aaaa....        ",
-"    .aaaa..         ",
+"   ..+aaaag.   ..   ",
+"  .+..aabagr. ..    ",
+"  .aa..abbgr...     ",
+"  .aab..bbgr..      ",
+"  .aaaa..ag..       ",
+"   .gggg....        ",
+"    .rrrr..         ",
 "     .......        ",
 "       .. ...       ",
 "      ..   ...      ",
@@ -1691,24 +1743,27 @@ const char *zoom_not_xpm[] =
 const char *zoom_undo_xpm[] =
 {
 /* width height num_colors chars_per_pixel */
-"    20    20        4            1",
+"    20    20        7            1",
 /* colors */
 "  none",
 ". c #000000",
 "+ c #c0c0c0",
 "a c #ffffff",
+"r c #af8f8f",
+"g c #8fff8f",
+"b c #8f8fff",
 /* pixels */
 "                    ",
 "                    ",
 "     .....          ",
 "    .+aaaa.         ",
-"   .++aaaaa.        ",
-"  .+++aaaaaa.       ",
-"  .aaaaaaaaa.       ",
-"  .aaaaaaaaa.       ",
-"  .aaaaaaaaa.       ",
-"   .aaaaaaa.        ",
-"    .aaaaa.         ",
+"   .++aaaag.        ",
+"  .+++aabagr.       ",
+"  .aaaaabbgr.       ",
+"  .aabbbbbgr.       ",
+"  .aaaabbgrr.       ",
+"   .gggggrr.        ",
+"    .rrrrr.         ",
 "     .......        ",
 "          ...       ",
 "                    ",
@@ -1725,24 +1780,27 @@ const char *zoom_undo_xpm[] =
 const char *zoom_out_xpm[] =
 {
 /* width height num_colors chars_per_pixel */
-"    20    20        4            1",
+"    20    20        7            1",
 /* colors */
 "  none",
 ". c #000000",
 "+ c #c0c0c0",
 "a c #ffffff",
+"r c #af8f8f",
+"g c #8fff8f",
+"b c #8f8fff",
 /* pixels */
 "                    ",
 "                    ",
 "     .....          ",
 "    .+aaaa.         ",
-"   .++aaaaa.        ",
-"  .+++aaaaaa.       ",
-"  .aaaaaaaaa.       ",
-"  .aaaaaaaaa.       ",
-"  .aaaaaaaaa.       ",
-"   .aaaaaaa.        ",
-"    .aaaaa.         ",
+"   .++aaaag.        ",
+"  .+++aabagr.       ",
+"  .aaaaabbgr.       ",
+"  .aabbbbbgr.       ",
+"  .aaaabbgrr.       ",
+"   .gggggrr.        ",
+"    .rrrrr.         ",
 "     .......        ",
 "          ...       ",
 "           ...      ",
@@ -1763,27 +1821,61 @@ const char *full_preview_area_xpm[] =
 /* colors */
 "  none",
 ". c #000000",
-"+ c #c0c0c0",
+"+ c #208020",
+"a c #ffffff",
+/* pixels */
+"                    ",
+" .................  ",
+" .+++++++++++++++.  ",
+" .+      .      +.  ",
+" .+     ...     +.  ",
+" .+    . . .    +.  ",
+" .+      .      +.  ",
+" .+      .      +.  ",
+" .+  .   .   .  +.  ",
+" .+ .    .    . +.  ",
+" .+.............+.  ",
+" .+ .    .    . +.  ",
+" .+  .   .   .  +.  ",
+" .+      .      +.  ",
+" .+    . . .    +.  ",
+" .+     ...     +.  ",
+" .+      .      +.  ",
+" .+++++++++++++++.  ",
+" .................  ",
+"                    ",
+};
+
+/* --------------------------------------------------- */
+
+const char *auto_raise_preview_area_xpm[] =
+{
+/* width height num_colors chars_per_pixel */
+"    20    20        4            1",
+/* colors */
+"  none",
+". c #000000",
+"+ c #208020",
 "a c #ffffff",
 /* pixels */
 "                    ",
 " .................  ",
 " .               .  ",
-" .       .       .  ",
-" .      ...      .  ",
-" .     . . .     .  ",
-" .       .       .  ",
-" .       .       .  ",
-" .   .   .   .   .  ",
-" .  .    .    .  .  ",
-" . ............. .  ",
-" .  .    .    .  .  ",
-" .   .   .   .   .  ",
-" .       .       .  ",
-" .       .       .  ",
-" .     . . .     .  ",
-" .      ...      .  ",
-" .       .       .  ",
+" .               .  ",
+" .  +++++++++++  .  ",
+" .  +    .    +  .  ",
+" .  +   ...   +  .  ",
+" .  +    .    +  .  ",
+" .  +    .    +  .  ",
+" .  + .  .  . +  .  ",
+" .  +.........+  .  ",
+" .  + .  .  . +  .  ",
+" .  +    .    +  .  ",
+" .  +    .    +  .  ",
+" .  +   ...   +  .  ",
+" .  +    .    +  .  ",
+" .  +++++++++++  .  ",
+" .               .  ",
 " .................  ",
 "                    ",
 };
@@ -1797,7 +1889,7 @@ const char *auto_select_preview_area_xpm[] =
 /* colors */
 "  none",
 ". c #000000",
-"+ c #c0c0c0",
+"+ c #208020",
 "a c #ffffff",
 /* pixels */
 "                    ",
@@ -1806,14 +1898,14 @@ const char *auto_select_preview_area_xpm[] =
 " .     . . .     .  ",
 " .      ...      .  ",
 " .       .       .  ",
-" .    .......    .  ",
-" .    .     .    .  ",
-" . .  .     .  . .  ",
-" .  . .     . .  .  ",
-" ......     ......  ",
-" .  . .     . .  .  ",
-" . .  .     .  . .  ",
-" .    .......    .  ",
+" .    +++++++    .  ",
+" .    +     +    .  ",
+" . .  +     +  . .  ",
+" .  . +     + .  .  ",
+" .....+     +.....  ",
+" .  . +     + .  .  ",
+" . .  +     +  . .  ",
+" .    +++++++    .  ",
 " .       .       .  ",
 " .      ...      .  ",
 " .     . . .     .  ",
@@ -1837,16 +1929,16 @@ const char *delete_images_xpm[] =
 "      ........      ",
 "     ..........     ",
 "    ............    ",
-"   ...   ..   ...   ",
-"   ...   ..   ...   ",
+"   ..    ..    ..   ",
+"   ..    ..    ..   ",
 "   ..............   ",
-"    .....  .....    ",
+"    ....    ....    ",
+"     ....  ....     ",
 "     ..........     ",
-"      ........      ",
-"       ......       ",
-"       ......       ",
-"..     ......     ..",
-"....    ....    ....",
+"     ..........     ",
+"     ..........     ",
+"..    ........    ..",
+"....   ......   ....",
 "   ...        ...   ",
 "      ...  ...      ",
 "        ....        ",
@@ -2556,3 +2648,39 @@ const unsigned char cursor_pipette_mask[] =
 };
 
 /* --------------------------------------------------- */
+
+const unsigned char cursor_autoraise_scanarea[] = 
+{
+   0xaa, 0x2a, 0xc1, 0x41, 0xe0, 0x03, 0x81, 0x40, 0x80, 0x00, 0x85, 0x50,
+   0x86, 0x30, 0x7f, 0x7f, 0x86, 0x30, 0x85, 0x50, 0x80, 0x00, 0x81, 0x40,
+   0xe0, 0x03, 0xc1, 0x41, 0xaa, 0x2a, 0x00, 0x00
+};
+
+/* --------------------------------------------------- */
+
+const unsigned char cursor_autoraise_scanarea_mask[] =
+{
+   0xea, 0x2b, 0xe1, 0x43, 0xf0, 0x07, 0xc1, 0x41, 0xc4, 0x11, 0xc7, 0x71,
+   0xff, 0x7f, 0x7f, 0x7f, 0xff, 0x7f, 0xc7, 0x71, 0xc4, 0x11, 0xc1, 0x41,
+   0xf0, 0x07, 0xe1, 0x43, 0xea, 0x2b, 0x00, 0x00
+};
+
+/* --------------------------------------------------- */
+
+const unsigned char cursor_zoom[] =
+{
+   0x00, 0x00, 0xf0, 0x01, 0x08, 0x02, 0x44, 0x04, 0x42, 0x08, 0x42, 0x08,
+   0xba, 0x0b, 0x42, 0x08, 0x42, 0x08, 0x44, 0x04, 0x08, 0x06, 0xf0, 0x0f,
+   0x00, 0x1c, 0x00, 0x38, 0x00, 0x70, 0x00, 0xe0
+};
+
+/* --------------------------------------------------- */
+
+const unsigned char cursor_zoom_mask[] =
+{
+   0xf0, 0x01, 0xf8, 0x03, 0x0c, 0x06, 0x46, 0x0c, 0x53, 0x19, 0xe3, 0x18,
+   0xbb, 0x1b, 0xe3, 0x18, 0x53, 0x19, 0x46, 0x0c, 0x0c, 0x0e, 0xf8, 0x1f,
+   0xf0, 0x3f, 0x00, 0x7c, 0x00, 0xf8, 0x00, 0xf0
+};
+
+/* --------------------------------------------------- */
